@@ -1,35 +1,35 @@
-// -*- c++ -*- (enables emacs c++ mode)
-//===========================================================================
-//
-// Copyright (C) 2003-2009 Yves Renard
-//
-// This file is a part of GETFEM++
-//
-// Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
-// under  the  terms  of the  GNU  Lesser General Public License as published
-// by  the  Free Software Foundation;  either version 2.1 of the License,  or
-// (at your option) any later version.
-// This program  is  distributed  in  the  hope  that it will be useful,  but
-// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-// or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
-// License for more details.
-// You  should  have received a copy of the GNU Lesser General Public License
-// along  with  this program;  if not, write to the Free Software Foundation,
-// Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-// As a special exception, you  may use  this file  as it is a part of a free
-// software  library  without  restriction.  Specifically,  if   other  files
-// instantiate  templates  or  use macros or inline functions from this file,
-// or  you compile this  file  and  link  it  with other files  to produce an
-// executable, this file  does  not  by itself cause the resulting executable
-// to be covered  by the GNU Lesser General Public License.  This   exception
-// does not  however  invalidate  any  other  reasons why the executable file
-// might be covered by the GNU Lesser General Public License.
-//
-//===========================================================================
+/* -*- c++ -*- (enables emacs c++ mode) */
+/*===========================================================================
+ 
+ Copyright (C) 2003-2015 Yves Renard
+ 
+ This file is a part of GETFEM++
+ 
+ Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+ under  the  terms  of the  GNU  Lesser General Public License as published
+ by  the  Free Software Foundation;  either version 3 of the License,  or
+ (at your option) any later version along with the GCC Runtime Library
+ Exception either version 3.1 or (at your option) any later version.
+ This program  is  distributed  in  the  hope  that it will be useful,  but
+ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ License and GCC Runtime Library Exception for more details.
+ You  should  have received a copy of the GNU Lesser General Public License
+ along  with  this program;  if not, write to the Free Software Foundation,
+ Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
+ 
+ As a special exception, you  may use  this file  as it is a part of a free
+ software  library  without  restriction.  Specifically,  if   other  files
+ instantiate  templates  or  use macros or inline functions from this file,
+ or  you compile this  file  and  link  it  with other files  to produce an
+ executable, this file  does  not  by itself cause the resulting executable
+ to be covered  by the GNU Lesser General Public License.  This   exception
+ does not  however  invalidate  any  other  reasons why the executable file
+ might be covered by the GNU Lesser General Public License.
+ 
+===========================================================================*/
 
 /**@file gmm_blas_interface.h
-   @author  Caroline Lecalvez, Caroline.Lecalvez@gmm.insa-tlse.fr,
    @author  Yves Renard <Yves.Renard@insa-lyon.fr>
    @date October 7, 2003.
    @brief gmm interface for fortran BLAS.
@@ -148,9 +148,15 @@ namespace gmm {
   /* ********************************************************************* */
   /* BLAS functions used.                                                  */
   /* ********************************************************************* */
-  extern "C" void daxpy_(const int *n, const double *alpha, const double *x, const int *incx, double *y, const int *incy);
   extern "C" {
-    void sgemm_(...); void dgemm_(...); void cgemm_(...); void zgemm_(...);
+    void daxpy_(const int *n, const double *alpha, const double *x,
+                const int *incx, double *y, const int *incy);
+    void dgemm_(const char *tA, const char *tB, const int *m,
+                const int *n, const int *k, const double *alpha,
+                const double *A, const int *ldA, const double *B,
+                const int *ldB, const double *beta, double *C,
+                const int *ldC);
+    void sgemm_(...); void cgemm_(...); void zgemm_(...);
     void sgemv_(...); void dgemv_(...); void cgemv_(...); void zgemv_(...);
     void strsv_(...); void dtrsv_(...); void ctrsv_(...); void ztrsv_(...);
     void saxpy_(...); /*void daxpy_(...); */void caxpy_(...); void zaxpy_(...);
@@ -212,20 +218,32 @@ namespace gmm {
          const_cast<std::vector<base_type > &>(*(linalg_origin(y_)));      \
          base_type b(y_.r)
 
-  dot_interface(dot_p1, dot_trans1, , dot_p2, dot_trans2, , sdot_ , BLAS_S)
-  dot_interface(dot_p1, dot_trans1, , dot_p2, dot_trans2, , ddot_ , BLAS_D)
-  dot_interface(dot_p1, dot_trans1, , dot_p2, dot_trans2, , cdotu_, BLAS_C)
-  dot_interface(dot_p1, dot_trans1, , dot_p2, dot_trans2, , zdotu_, BLAS_Z)
+  dot_interface(dot_p1, dot_trans1, (BLAS_S), dot_p2, dot_trans2, (BLAS_S),
+		sdot_ , BLAS_S)
+  dot_interface(dot_p1, dot_trans1, (BLAS_D), dot_p2, dot_trans2, (BLAS_D),
+		ddot_ , BLAS_D)
+  dot_interface(dot_p1, dot_trans1, (BLAS_C), dot_p2, dot_trans2, (BLAS_C),
+		cdotu_, BLAS_C)
+  dot_interface(dot_p1, dot_trans1, (BLAS_Z), dot_p2, dot_trans2, (BLAS_Z),
+		zdotu_, BLAS_Z)
   
-  dot_interface(dot_p1_s, dot_trans1_s, a*, dot_p2, dot_trans2, ,sdot_ ,BLAS_S)
-  dot_interface(dot_p1_s, dot_trans1_s, a*, dot_p2, dot_trans2, ,ddot_ ,BLAS_D)
-  dot_interface(dot_p1_s, dot_trans1_s, a*, dot_p2, dot_trans2, ,cdotu_,BLAS_C)
-  dot_interface(dot_p1_s, dot_trans1_s, a*, dot_p2, dot_trans2, ,zdotu_,BLAS_Z)
+  dot_interface(dot_p1_s, dot_trans1_s, a*, dot_p2, dot_trans2, (BLAS_S),
+		sdot_ ,BLAS_S)
+  dot_interface(dot_p1_s, dot_trans1_s, a*, dot_p2, dot_trans2, (BLAS_D),
+		ddot_ ,BLAS_D)
+  dot_interface(dot_p1_s, dot_trans1_s, a*, dot_p2, dot_trans2, (BLAS_C),
+		cdotu_,BLAS_C)
+  dot_interface(dot_p1_s, dot_trans1_s, a*, dot_p2, dot_trans2, (BLAS_Z),
+		zdotu_,BLAS_Z)
   
-  dot_interface(dot_p1, dot_trans1, , dot_p2_s, dot_trans2_s, b*,sdot_ ,BLAS_S)
-  dot_interface(dot_p1, dot_trans1, , dot_p2_s, dot_trans2_s, b*,ddot_ ,BLAS_D)
-  dot_interface(dot_p1, dot_trans1, , dot_p2_s, dot_trans2_s, b*,cdotu_,BLAS_C)
-  dot_interface(dot_p1, dot_trans1, , dot_p2_s, dot_trans2_s, b*,zdotu_,BLAS_Z)
+  dot_interface(dot_p1, dot_trans1, (BLAS_S), dot_p2_s, dot_trans2_s, b*,
+		sdot_ ,BLAS_S)
+  dot_interface(dot_p1, dot_trans1, (BLAS_D), dot_p2_s, dot_trans2_s, b*,
+		ddot_ ,BLAS_D)
+  dot_interface(dot_p1, dot_trans1, (BLAS_C), dot_p2_s, dot_trans2_s, b*,
+		cdotu_,BLAS_C)
+  dot_interface(dot_p1, dot_trans1, (BLAS_Z), dot_p2_s, dot_trans2_s, b*,
+		  zdotu_,BLAS_Z)
 
   dot_interface(dot_p1_s,dot_trans1_s,a*,dot_p2_s,dot_trans2_s,b*,sdot_ ,
 		BLAS_S)
@@ -267,28 +285,32 @@ namespace gmm {
          const_cast<std::vector<base_type > &>(*(linalg_origin(y_)));      \
          base_type b(gmm::conj(y_.r))
 
-  dotc_interface(dotc_p1, dotc_trans1, , dotc_p2, dotc_trans2, ,sdot_ ,BLAS_S)
-  dotc_interface(dotc_p1, dotc_trans1, , dotc_p2, dotc_trans2, ,ddot_ ,BLAS_D)
-  dotc_interface(dotc_p1, dotc_trans1, , dotc_p2, dotc_trans2, ,cdotc_,BLAS_C)
-  dotc_interface(dotc_p1, dotc_trans1, , dotc_p2, dotc_trans2, ,zdotc_,BLAS_Z)
+  dotc_interface(dotc_p1, dotc_trans1, (BLAS_S), dotc_p2, dotc_trans2,
+		 (BLAS_S),sdot_ ,BLAS_S)
+  dotc_interface(dotc_p1, dotc_trans1, (BLAS_D), dotc_p2, dotc_trans2,
+		 (BLAS_D),ddot_ ,BLAS_D)
+  dotc_interface(dotc_p1, dotc_trans1, (BLAS_C), dotc_p2, dotc_trans2,
+		 (BLAS_C),cdotc_,BLAS_C)
+  dotc_interface(dotc_p1, dotc_trans1, (BLAS_Z), dotc_p2, dotc_trans2,
+		 (BLAS_Z),zdotc_,BLAS_Z)
   
-  dotc_interface(dotc_p1_s, dotc_trans1_s, a*, dotc_p2, dotc_trans2, ,sdot_,
-		 BLAS_S)
-  dotc_interface(dotc_p1_s, dotc_trans1_s, a*, dotc_p2, dotc_trans2, ,ddot_ ,
-		 BLAS_D)
-  dotc_interface(dotc_p1_s, dotc_trans1_s, a*, dotc_p2, dotc_trans2, ,cdotc_,
-		 BLAS_C)
-  dotc_interface(dotc_p1_s, dotc_trans1_s, a*, dotc_p2, dotc_trans2, ,zdotc_,
-		 BLAS_Z)
+  dotc_interface(dotc_p1_s, dotc_trans1_s, a*, dotc_p2, dotc_trans2,
+		 (BLAS_S),sdot_, BLAS_S)
+  dotc_interface(dotc_p1_s, dotc_trans1_s, a*, dotc_p2, dotc_trans2,
+		 (BLAS_D),ddot_ , BLAS_D)
+  dotc_interface(dotc_p1_s, dotc_trans1_s, a*, dotc_p2, dotc_trans2,
+		 (BLAS_C),cdotc_, BLAS_C)
+  dotc_interface(dotc_p1_s, dotc_trans1_s, a*, dotc_p2, dotc_trans2,
+		 (BLAS_Z),zdotc_, BLAS_Z)
   
-  dotc_interface(dotc_p1, dotc_trans1, , dotc_p2_s, dotc_trans2_s, b*,sdot_ ,
-		 BLAS_S)
-  dotc_interface(dotc_p1, dotc_trans1, , dotc_p2_s, dotc_trans2_s, b*,ddot_ ,
-		 BLAS_D)
-  dotc_interface(dotc_p1, dotc_trans1, , dotc_p2_s, dotc_trans2_s, b*,cdotc_,
-		 BLAS_C)
-  dotc_interface(dotc_p1, dotc_trans1, , dotc_p2_s, dotc_trans2_s, b*,zdotc_,
-		 BLAS_Z)
+  dotc_interface(dotc_p1, dotc_trans1, (BLAS_S), dotc_p2_s, dotc_trans2_s,
+		 b*,sdot_ , BLAS_S)
+  dotc_interface(dotc_p1, dotc_trans1, (BLAS_D), dotc_p2_s, dotc_trans2_s,
+		 b*,ddot_ , BLAS_D)
+  dotc_interface(dotc_p1, dotc_trans1, (BLAS_C), dotc_p2_s, dotc_trans2_s,
+		 b*,cdotc_, BLAS_C)
+  dotc_interface(dotc_p1, dotc_trans1, (BLAS_Z), dotc_p2_s, dotc_trans2_s,
+		   b*,zdotc_, BLAS_Z)
 
   dotc_interface(dotc_p1_s,dotc_trans1_s,a*,dotc_p2_s,dotc_trans2_s,b*,sdot_ ,
 		 BLAS_S)
@@ -637,7 +659,7 @@ namespace gmm {
 
 # define gemm_interface_tn(blas_name, base_type, is_const)                 \
   inline void mult_spec(                                                   \
-         const transposed_col_ref<is_const dense_matrix<base_type > *> &A_,\
+         const transposed_col_ref<is_const<base_type > *> &A_,\
          const dense_matrix<base_type > &B,                                \
          dense_matrix<base_type > &C, rcmult) {                            \
     GMMLAPACK_TRACE("gemm_interface_tn");                                  \
@@ -653,14 +675,14 @@ namespace gmm {
     else gmm::clear(C);                                                    \
   }
 
-  gemm_interface_tn(sgemm_, BLAS_S,)
-  gemm_interface_tn(dgemm_, BLAS_D,)
-  gemm_interface_tn(cgemm_, BLAS_C,)
-  gemm_interface_tn(zgemm_, BLAS_Z,)
-  gemm_interface_tn(sgemm_, BLAS_S, const)
-  gemm_interface_tn(dgemm_, BLAS_D, const)
-  gemm_interface_tn(cgemm_, BLAS_C, const)
-  gemm_interface_tn(zgemm_, BLAS_Z, const)
+  gemm_interface_tn(sgemm_, BLAS_S, dense_matrix)
+  gemm_interface_tn(dgemm_, BLAS_D, dense_matrix)
+  gemm_interface_tn(cgemm_, BLAS_C, dense_matrix)
+  gemm_interface_tn(zgemm_, BLAS_Z, dense_matrix)
+  gemm_interface_tn(sgemm_, BLAS_S, const dense_matrix)
+  gemm_interface_tn(dgemm_, BLAS_D, const dense_matrix)
+  gemm_interface_tn(cgemm_, BLAS_C, const dense_matrix)
+  gemm_interface_tn(zgemm_, BLAS_Z, const dense_matrix)
 
   /* ********************************************************************* */
   /* dense matrix x transposed(dense matrix) multiplication.               */
@@ -668,7 +690,7 @@ namespace gmm {
 
 # define gemm_interface_nt(blas_name, base_type, is_const)                 \
   inline void mult_spec(const dense_matrix<base_type > &A,                 \
-         const transposed_col_ref<is_const dense_matrix<base_type > *> &B_,\
+         const transposed_col_ref<is_const<base_type > *> &B_,\
          dense_matrix<base_type > &C, r_mult) {                            \
     GMMLAPACK_TRACE("gemm_interface_nt");                                  \
     dense_matrix<base_type > &B                                            \
@@ -684,14 +706,14 @@ namespace gmm {
     else gmm::clear(C);                                                    \
   }
 
-  gemm_interface_nt(sgemm_, BLAS_S,)
-  gemm_interface_nt(dgemm_, BLAS_D,)
-  gemm_interface_nt(cgemm_, BLAS_C,)
-  gemm_interface_nt(zgemm_, BLAS_Z,)
-  gemm_interface_nt(sgemm_, BLAS_S, const)
-  gemm_interface_nt(dgemm_, BLAS_D, const)
-  gemm_interface_nt(cgemm_, BLAS_C, const)
-  gemm_interface_nt(zgemm_, BLAS_Z, const)
+  gemm_interface_nt(sgemm_, BLAS_S, dense_matrix)
+  gemm_interface_nt(dgemm_, BLAS_D, dense_matrix)
+  gemm_interface_nt(cgemm_, BLAS_C, dense_matrix)
+  gemm_interface_nt(zgemm_, BLAS_Z, dense_matrix)
+  gemm_interface_nt(sgemm_, BLAS_S, const dense_matrix)
+  gemm_interface_nt(dgemm_, BLAS_D, const dense_matrix)
+  gemm_interface_nt(cgemm_, BLAS_C, const dense_matrix)
+  gemm_interface_nt(zgemm_, BLAS_Z, const dense_matrix)
 
   /* ********************************************************************* */
   /* transposed(dense matrix) x transposed(dense matrix) multiplication.   */
@@ -699,8 +721,8 @@ namespace gmm {
 
 # define gemm_interface_tt(blas_name, base_type, isA_const, isB_const)     \
   inline void mult_spec(                                                   \
-        const transposed_col_ref<isA_const dense_matrix<base_type > *> &A_,\
-        const transposed_col_ref<isB_const dense_matrix<base_type > *> &B_,\
+        const transposed_col_ref<isA_const <base_type > *> &A_,\
+        const transposed_col_ref<isB_const <base_type > *> &B_,\
         dense_matrix<base_type > &C, r_mult) {                             \
     GMMLAPACK_TRACE("gemm_interface_tt");                                  \
     dense_matrix<base_type > &A                                            \
@@ -717,22 +739,22 @@ namespace gmm {
     else gmm::clear(C);                                                    \
   }
 
-  gemm_interface_tt(sgemm_, BLAS_S,,)
-  gemm_interface_tt(dgemm_, BLAS_D,,)
-  gemm_interface_tt(cgemm_, BLAS_C,,)
-  gemm_interface_tt(zgemm_, BLAS_Z,,)
-  gemm_interface_tt(sgemm_, BLAS_S, const,)
-  gemm_interface_tt(dgemm_, BLAS_D, const,)
-  gemm_interface_tt(cgemm_, BLAS_C, const,)
-  gemm_interface_tt(zgemm_, BLAS_Z, const,)
-  gemm_interface_tt(sgemm_, BLAS_S,, const)
-  gemm_interface_tt(dgemm_, BLAS_D,, const)
-  gemm_interface_tt(cgemm_, BLAS_C,, const)
-  gemm_interface_tt(zgemm_, BLAS_Z,, const)
-  gemm_interface_tt(sgemm_, BLAS_S, const, const)
-  gemm_interface_tt(dgemm_, BLAS_D, const, const)
-  gemm_interface_tt(cgemm_, BLAS_C, const, const)
-  gemm_interface_tt(zgemm_, BLAS_Z, const, const)
+  gemm_interface_tt(sgemm_, BLAS_S, dense_matrix, dense_matrix)
+  gemm_interface_tt(dgemm_, BLAS_D, dense_matrix, dense_matrix)
+  gemm_interface_tt(cgemm_, BLAS_C, dense_matrix, dense_matrix)
+  gemm_interface_tt(zgemm_, BLAS_Z, dense_matrix, dense_matrix)
+  gemm_interface_tt(sgemm_, BLAS_S, const dense_matrix, dense_matrix)
+  gemm_interface_tt(dgemm_, BLAS_D, const dense_matrix, dense_matrix)
+  gemm_interface_tt(cgemm_, BLAS_C, const dense_matrix, dense_matrix)
+  gemm_interface_tt(zgemm_, BLAS_Z, const dense_matrix, dense_matrix)
+  gemm_interface_tt(sgemm_, BLAS_S, dense_matrix, const dense_matrix)
+  gemm_interface_tt(dgemm_, BLAS_D, dense_matrix, const dense_matrix)
+  gemm_interface_tt(cgemm_, BLAS_C, dense_matrix, const dense_matrix)
+  gemm_interface_tt(zgemm_, BLAS_Z, dense_matrix, const dense_matrix)
+  gemm_interface_tt(sgemm_, BLAS_S, const dense_matrix, const dense_matrix)
+  gemm_interface_tt(dgemm_, BLAS_D, const dense_matrix, const dense_matrix)
+  gemm_interface_tt(cgemm_, BLAS_C, const dense_matrix, const dense_matrix)
+  gemm_interface_tt(zgemm_, BLAS_Z, const dense_matrix, const dense_matrix)
 
 
   /* ********************************************************************* */
