@@ -1,32 +1,33 @@
-// -*- c++ -*- (enables emacs c++ mode)
-//===========================================================================
-//
-// Copyright (C) 2002-2008 Yves Renard
-//
-// This file is a part of GETFEM++
-//
-// Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
-// under  the  terms  of the  GNU  Lesser General Public License as published
-// by  the  Free Software Foundation;  either version 2.1 of the License,  or
-// (at your option) any later version.
-// This program  is  distributed  in  the  hope  that it will be useful,  but
-// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-// or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
-// License for more details.
-// You  should  have received a copy of the GNU Lesser General Public License
-// along  with  this program;  if not, write to the Free Software Foundation,
-// Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-// As a special exception, you  may use  this file  as it is a part of a free
-// software  library  without  restriction.  Specifically,  if   other  files
-// instantiate  templates  or  use macros or inline functions from this file,
-// or  you compile this  file  and  link  it  with other files  to produce an
-// executable, this file  does  not  by itself cause the resulting executable
-// to be covered  by the GNU Lesser General Public License.  This   exception
-// does not  however  invalidate  any  other  reasons why the executable file
-// might be covered by the GNU Lesser General Public License.
-//
-//===========================================================================
+/* -*- c++ -*- (enables emacs c++ mode) */
+/*===========================================================================
+ 
+ Copyright (C) 2002-2015 Yves Renard
+ 
+ This file is a part of GETFEM++
+ 
+ Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+ under  the  terms  of the  GNU  Lesser General Public License as published
+ by  the  Free Software Foundation;  either version 3 of the License,  or
+ (at your option) any later version along with the GCC Runtime Library
+ Exception either version 3.1 or (at your option) any later version.
+ This program  is  distributed  in  the  hope  that it will be useful,  but
+ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ License and GCC Runtime Library Exception for more details.
+ You  should  have received a copy of the GNU Lesser General Public License
+ along  with  this program;  if not, write to the Free Software Foundation,
+ Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
+ 
+ As a special exception, you  may use  this file  as it is a part of a free
+ software  library  without  restriction.  Specifically,  if   other  files
+ instantiate  templates  or  use macros or inline functions from this file,
+ or  you compile this  file  and  link  it  with other files  to produce an
+ executable, this file  does  not  by itself cause the resulting executable
+ to be covered  by the GNU Lesser General Public License.  This   exception
+ does not  however  invalidate  any  other  reasons why the executable file
+ might be covered by the GNU Lesser General Public License.
+ 
+===========================================================================*/
 
 /**@file gmm_def.h
    @author  Yves Renard <Yves.Renard@insa-lyon.fr>
@@ -359,6 +360,7 @@ namespace gmm {
   template <typename T> inline T pos(T a) { return (a < T(0)) ? T(0) : a; }
   template <typename T> inline T neg(T a) { return (a < T(0)) ? T(-a) : T(0); }
   template <typename T> inline T sgn(T a) { return (a < T(0)) ? T(-1) : T(1); }
+  template <typename T> inline T Heaviside(T a) { return (a < T(0)) ? T(0) : T(1); }
   inline double random() { return double(rand())/(RAND_MAX+0.5); }
   template <typename T> inline T random(T)
   { return T(rand()*2.0)/(T(RAND_MAX)+T(1)/T(2)) - T(1); }
@@ -959,7 +961,7 @@ namespace gmm {
       if (numeric_limits<T>::is_specialized)
 	tol = numeric_limits<T>::epsilon();
       else {
-	int i=sizeof(T)/4; while(i-- > 0) tol*=T(1E-8); 
+	int i=int(sizeof(T)/4); while(i-- > 0) tol*=T(1E-8); 
 	GMM_WARNING1("The numeric type " << typeid(T).name()
 		    << " has no numeric_limits defined !!\n"
 		    << "Taking " << tol << " as default tolerance");
@@ -1049,7 +1051,7 @@ namespace gmm {
     typename linalg_traits<L>::const_iterator it = vect_const_begin(l),
       ite = vect_const_end(l);
     for (; it != ite; ++it) 
-      o << " (r" << it.index() << "," << cast_char(*it) << ")";
+      o << " (r" << it.index() << ", " << cast_char(*it) << ")";
   }
 
   template <typename L> void write(std::ostream &o, const L &l,
