@@ -1,32 +1,33 @@
-// -*- c++ -*- (enables emacs c++ mode)
-//===========================================================================
-//
-// Copyright (C) 2003-2008 Yves Renard
-//
-// This file is a part of GETFEM++
-//
-// Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
-// under  the  terms  of the  GNU  Lesser General Public License as published
-// by  the  Free Software Foundation;  either version 2.1 of the License,  or
-// (at your option) any later version.
-// This program  is  distributed  in  the  hope  that it will be useful,  but
-// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-// or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
-// License for more details.
-// You  should  have received a copy of the GNU Lesser General Public License
-// along  with  this program;  if not, write to the Free Software Foundation,
-// Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-// As a special exception, you  may use  this file  as it is a part of a free
-// software  library  without  restriction.  Specifically,  if   other  files
-// instantiate  templates  or  use macros or inline functions from this file,
-// or  you compile this  file  and  link  it  with other files  to produce an
-// executable, this file  does  not  by itself cause the resulting executable
-// to be covered  by the GNU Lesser General Public License.  This   exception
-// does not  however  invalidate  any  other  reasons why the executable file
-// might be covered by the GNU Lesser General Public License.
-//
-//===========================================================================
+/* -*- c++ -*- (enables emacs c++ mode) */
+/*===========================================================================
+ 
+ Copyright (C) 2003-2015 Yves Renard
+ 
+ This file is a part of GETFEM++
+ 
+ Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+ under  the  terms  of the  GNU  Lesser General Public License as published
+ by  the  Free Software Foundation;  either version 3 of the License,  or
+ (at your option) any later version along with the GCC Runtime Library
+ Exception either version 3.1 or (at your option) any later version.
+ This program  is  distributed  in  the  hope  that it will be useful,  but
+ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ License and GCC Runtime Library Exception for more details.
+ You  should  have received a copy of the GNU Lesser General Public License
+ along  with  this program;  if not, write to the Free Software Foundation,
+ Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
+ 
+ As a special exception, you  may use  this file  as it is a part of a free
+ software  library  without  restriction.  Specifically,  if   other  files
+ instantiate  templates  or  use macros or inline functions from this file,
+ or  you compile this  file  and  link  it  with other files  to produce an
+ executable, this file  does  not  by itself cause the resulting executable
+ to be covered  by the GNU Lesser General Public License.  This   exception
+ does not  however  invalidate  any  other  reasons why the executable file
+ might be covered by the GNU Lesser General Public License.
+ 
+===========================================================================*/
 
 /**@file gmm_dense_qr.h
    @author  Caroline Lecalvez, Caroline.Lecalvez@gmm.insa-tlse.fr, Yves Renard <Yves.Renard@insa-lyon.fr>
@@ -45,7 +46,7 @@ namespace gmm {
      QR factorization using Householder method (complex and real version).
   */
   template <typename MAT1>
-  void qr_factor(const MAT1 &A_) { 
+  void qr_factor(const MAT1 &A_) {
     MAT1 &A = const_cast<MAT1 &>(A_);
     typedef typename linalg_traits<MAT1>::value_type value_type;
 
@@ -66,12 +67,12 @@ namespace gmm {
     }
   }
 
-  
+
   // QR comes from QR_factor(QR) where the upper triangular part stands for R
   // and the lower part contains the Householder reflectors.
   // A <- AQ
   template <typename MAT1, typename MAT2>
-  void apply_house_right(const MAT1 &QR, const MAT2 &A_) { 
+  void apply_house_right(const MAT1 &QR, const MAT2 &A_) {
     MAT2 &A = const_cast<MAT2 &>(A_);
     typedef typename linalg_traits<MAT1>::value_type T;
     size_type m = mat_nrows(QR), n = mat_ncols(QR);
@@ -83,7 +84,7 @@ namespace gmm {
       V.resize(m-j);
       for (size_type i = j+1; i < m; ++i) V[i-j] = QR(i, j);
       col_house_update(sub_matrix(A, sub_interval(0, mat_nrows(A)),
-				  sub_interval(j, m-j)), V, W);
+                                  sub_interval(j, m-j)), V, W);
     }
   }
 
@@ -91,7 +92,7 @@ namespace gmm {
   // and the lower part contains the Householder reflectors.
   // A <- Q*A
   template <typename MAT1, typename MAT2>
-  void apply_house_left(const MAT1 &QR, const MAT2 &A_) { 
+  void apply_house_left(const MAT1 &QR, const MAT2 &A_) {
     MAT2 &A = const_cast<MAT2 &>(A_);
     typedef typename linalg_traits<MAT1>::value_type T;
     size_type m = mat_nrows(QR), n = mat_ncols(QR);
@@ -103,20 +104,20 @@ namespace gmm {
       V.resize(m-j);
       for (size_type i = j+1; i < m; ++i) V[i-j] = QR(i, j);
       row_house_update(sub_matrix(A, sub_interval(j, m-j),
-				  sub_interval(0, mat_ncols(A))), V, W);
+                                  sub_interval(0, mat_ncols(A))), V, W);
     }
-  }  
+  }
 
   /** Compute the QR factorization, where Q is assembled. */
   template <typename MAT1, typename MAT2, typename MAT3>
-    void qr_factor(const MAT1 &A, const MAT2 &QQ, const MAT3 &RR) { 
-    MAT2 &Q = const_cast<MAT2 &>(QQ); MAT3 &R = const_cast<MAT3 &>(RR); 
+    void qr_factor(const MAT1 &A, const MAT2 &QQ, const MAT3 &RR) {
+    MAT2 &Q = const_cast<MAT2 &>(QQ); MAT3 &R = const_cast<MAT3 &>(RR);
     typedef typename linalg_traits<MAT1>::value_type value_type;
 
     size_type m = mat_nrows(A), n = mat_ncols(A);
     GMM_ASSERT2(m >= n, "dimensions mismatch");
     gmm::copy(A, Q);
-    
+
     std::vector<value_type> W(m);
     dense_matrix<value_type> VV(m, n);
 
@@ -127,22 +128,22 @@ namespace gmm {
       house_vector(sub_vector(mat_col(VV,j), SUBI));
 
       row_house_update(sub_matrix(Q, SUBI, SUBJ),
-		       sub_vector(mat_col(VV,j), SUBI), sub_vector(W, SUBJ));
+                       sub_vector(mat_col(VV,j), SUBI), sub_vector(W, SUBJ));
     }
 
     gmm::copy(sub_matrix(Q, sub_interval(0, n), sub_interval(0, n)), R);
     gmm::copy(identity_matrix(), Q);
-    
+
     for (size_type j = n-1; j != size_type(-1); --j) {
       sub_interval SUBI(j, m-j), SUBJ(j, n-j);
-      row_house_update(sub_matrix(Q, SUBI, SUBJ), 
-		       sub_vector(mat_col(VV,j), SUBI), sub_vector(W, SUBJ));
+      row_house_update(sub_matrix(Q, SUBI, SUBJ),
+                       sub_vector(mat_col(VV,j), SUBI), sub_vector(W, SUBJ));
     }
   }
 
   ///@cond DOXY_SHOW_ALL_FUNCTIONS
-  template <typename TA, typename TV, typename Ttol, 
-	    typename MAT, typename VECT>
+  template <typename TA, typename TV, typename Ttol,
+            typename MAT, typename VECT>
   void extract_eig(const MAT &A, VECT &V, Ttol tol, TA, TV) {
     size_type n = mat_nrows(A);
     if (n == 0) return;
@@ -150,57 +151,57 @@ namespace gmm {
     Ttol tol_i = tol * gmm::abs(A(0,0)), tol_cplx = tol_i;
     for (size_type i = 0; i < n; ++i) {
       if (i < n-1) {
-	tol_i = (gmm::abs(A(i,i))+gmm::abs(A(i+1,i+1)))*tol;
-	tol_cplx = std::max(tol_cplx, tol_i);
+        tol_i = (gmm::abs(A(i,i))+gmm::abs(A(i+1,i+1)))*tol;
+        tol_cplx = std::max(tol_cplx, tol_i);
       }
       if ((i < n-1) && gmm::abs(A(i+1,i)) >= tol_i) {
-	TA tr = A(i,i) + A(i+1, i+1);
-	TA det = A(i,i)*A(i+1, i+1) - A(i,i+1)*A(i+1, i);
-	TA delta = tr*tr - TA(4) * det;
-	if (delta < -tol_cplx) {
-	  GMM_WARNING1("A complex eigenvalue has been detected : "
-		      << std::complex<TA>(tr/TA(2), gmm::sqrt(-delta)/TA(2)));
-	  V[i] = V[i+1] = tr / TA(2);
-	}
-	else {
-	  delta = std::max(TA(0), delta);
-	  V[i  ] = TA(tr + gmm::sqrt(delta))/ TA(2);
-	  V[i+1] = TA(tr -  gmm::sqrt(delta))/ TA(2);
-	}
-	++i;
+        TA tr = A(i,i) + A(i+1, i+1);
+        TA det = A(i,i)*A(i+1, i+1) - A(i,i+1)*A(i+1, i);
+        TA delta = tr*tr - TA(4) * det;
+        if (delta < -tol_cplx) {
+          GMM_WARNING1("A complex eigenvalue has been detected : "
+                      << std::complex<TA>(tr/TA(2), gmm::sqrt(-delta)/TA(2)));
+          V[i] = V[i+1] = tr / TA(2);
+        }
+        else {
+          delta = std::max(TA(0), delta);
+          V[i  ] = TA(tr + gmm::sqrt(delta))/ TA(2);
+          V[i+1] = TA(tr -  gmm::sqrt(delta))/ TA(2);
+        }
+        ++i;
       }
       else
-	V[i] = TV(A(i,i));
+        V[i] = TV(A(i,i));
     }
   }
 
-  template <typename TA, typename TV, typename Ttol, 
-	    typename MAT, typename VECT>
+  template <typename TA, typename TV, typename Ttol,
+            typename MAT, typename VECT>
   void extract_eig(const MAT &A, VECT &V, Ttol tol, TA, std::complex<TV>) {
     size_type n = mat_nrows(A);
     tol *= Ttol(2);
     for (size_type i = 0; i < n; ++i)
       if ((i == n-1) ||
-	  gmm::abs(A(i+1,i)) < (gmm::abs(A(i,i))+gmm::abs(A(i+1,i+1)))*tol)
-	V[i] = std::complex<TV>(A(i,i));
+          gmm::abs(A(i+1,i)) < (gmm::abs(A(i,i))+gmm::abs(A(i+1,i+1)))*tol)
+        V[i] = std::complex<TV>(A(i,i));
       else {
-	TA tr = A(i,i) + A(i+1, i+1);
-	TA det = A(i,i)*A(i+1, i+1) - A(i,i+1)*A(i+1, i);
-	TA delta = tr*tr - TA(4) * det;
-	if (delta < TA(0)) {
-	  V[i] = std::complex<TV>(tr / TA(2), gmm::sqrt(-delta) / TA(2));
-	  V[i+1] = std::complex<TV>(tr / TA(2), -gmm::sqrt(-delta)/ TA(2));
-	}
-	else {
-	  V[i  ] = TA(tr + gmm::sqrt(delta)) / TA(2);
-	  V[i+1] = TA(tr -  gmm::sqrt(delta)) / TA(2);
-	}
-	++i;
+        TA tr = A(i,i) + A(i+1, i+1);
+        TA det = A(i,i)*A(i+1, i+1) - A(i,i+1)*A(i+1, i);
+        TA delta = tr*tr - TA(4) * det;
+        if (delta < TA(0)) {
+          V[i] = std::complex<TV>(tr / TA(2), gmm::sqrt(-delta) / TA(2));
+          V[i+1] = std::complex<TV>(tr / TA(2), -gmm::sqrt(-delta)/ TA(2));
+        }
+        else {
+          V[i  ] = TA(tr + gmm::sqrt(delta)) / TA(2);
+          V[i+1] = TA(tr -  gmm::sqrt(delta)) / TA(2);
+        }
+        ++i;
       }
   }
 
   template <typename TA, typename TV, typename Ttol,
-	    typename MAT, typename VECT>
+            typename MAT, typename VECT>
   void extract_eig(const MAT &A, VECT &V, Ttol tol, std::complex<TA>, TV) {
     typedef std::complex<TA> T;
     size_type n = mat_nrows(A);
@@ -209,50 +210,50 @@ namespace gmm {
     Ttol tol_i = tol * gmm::abs(A(0,0)), tol_cplx = tol_i;
     for (size_type i = 0; i < n; ++i) {
       if (i < n-1) {
-	tol_i = (gmm::abs(A(i,i))+gmm::abs(A(i+1,i+1)))*tol;
-	tol_cplx = std::max(tol_cplx, tol_i);
+        tol_i = (gmm::abs(A(i,i))+gmm::abs(A(i+1,i+1)))*tol;
+        tol_cplx = std::max(tol_cplx, tol_i);
       }
       if ((i == n-1) || gmm::abs(A(i+1,i)) < tol_i) {
-	if (gmm::abs(std::imag(A(i,i))) > tol_cplx)
-	  GMM_WARNING1("A complex eigenvalue has been detected : "
-		      << T(A(i,i)) << " : "  << gmm::abs(std::imag(A(i,i)))
-		      / gmm::abs(std::real(A(i,i))) << " : " << tol_cplx);
-	V[i] = std::real(A(i,i));
+        if (gmm::abs(std::imag(A(i,i))) > tol_cplx)
+          GMM_WARNING1("A complex eigenvalue has been detected : "
+                      << T(A(i,i)) << " : "  << gmm::abs(std::imag(A(i,i)))
+                      / gmm::abs(std::real(A(i,i))) << " : " << tol_cplx);
+        V[i] = std::real(A(i,i));
       }
       else {
-	T tr = A(i,i) + A(i+1, i+1);
-	T det = A(i,i)*A(i+1, i+1) - A(i,i+1)*A(i+1, i);
-	T delta = tr*tr - TA(4) * det;
-	T a1 = (tr + gmm::sqrt(delta)) / TA(2);
-	T a2 = (tr - gmm::sqrt(delta)) / TA(2);
-	if (gmm::abs(std::imag(a1)) > tol_cplx)
-	  GMM_WARNING1("A complex eigenvalue has been detected : " << a1);
-	if (gmm::abs(std::imag(a2)) > tol_cplx)
-	  GMM_WARNING1("A complex eigenvalue has been detected : " << a2);
+        T tr = A(i,i) + A(i+1, i+1);
+        T det = A(i,i)*A(i+1, i+1) - A(i,i+1)*A(i+1, i);
+        T delta = tr*tr - TA(4) * det;
+        T a1 = (tr + gmm::sqrt(delta)) / TA(2);
+        T a2 = (tr - gmm::sqrt(delta)) / TA(2);
+        if (gmm::abs(std::imag(a1)) > tol_cplx)
+          GMM_WARNING1("A complex eigenvalue has been detected : " << a1);
+        if (gmm::abs(std::imag(a2)) > tol_cplx)
+          GMM_WARNING1("A complex eigenvalue has been detected : " << a2);
 
-	V[i] = std::real(a1); V[i+1] = std::real(a2);
-	++i;
+        V[i] = std::real(a1); V[i+1] = std::real(a2);
+        ++i;
       }
     }
   }
 
   template <typename TA, typename TV, typename Ttol,
-	    typename MAT, typename VECT>
+            typename MAT, typename VECT>
   void extract_eig(const MAT &A, VECT &V, Ttol tol,
-		   std::complex<TA>, std::complex<TV>) {
+                   std::complex<TA>, std::complex<TV>) {
     size_type n = mat_nrows(A);
     tol *= Ttol(2);
     for (size_type i = 0; i < n; ++i)
       if ((i == n-1) ||
-	  gmm::abs(A(i+1,i)) < (gmm::abs(A(i,i))+gmm::abs(A(i+1,i+1)))*tol)
-	V[i] = std::complex<TV>(A(i,i));
+          gmm::abs(A(i+1,i)) < (gmm::abs(A(i,i))+gmm::abs(A(i+1,i+1)))*tol)
+        V[i] = std::complex<TV>(A(i,i));
       else {
-	std::complex<TA> tr = A(i,i) + A(i+1, i+1);
-	std::complex<TA> det = A(i,i)*A(i+1, i+1) - A(i,i+1)*A(i+1, i);
-	std::complex<TA> delta = tr*tr - TA(4) * det;
-	V[i] = (tr + gmm::sqrt(delta)) / TA(2);
-	V[i+1] = (tr - gmm::sqrt(delta)) / TA(2);
-	++i;
+        std::complex<TA> tr = A(i,i) + A(i+1, i+1);
+        std::complex<TA> det = A(i,i)*A(i+1, i+1) - A(i,i+1)*A(i+1, i);
+        std::complex<TA> delta = tr*tr - TA(4) * det;
+        V[i] = (tr + gmm::sqrt(delta)) / TA(2);
+        V[i+1] = (tr - gmm::sqrt(delta)) / TA(2);
+        ++i;
       }
   }
 
@@ -263,8 +264,8 @@ namespace gmm {
   template <typename MAT, typename Ttol, typename VECT> inline
   void extract_eig(const MAT &A, const VECT &V, Ttol tol) {
     extract_eig(A, const_cast<VECT&>(V), tol,
-		typename linalg_traits<MAT>::value_type(),
-		typename linalg_traits<VECT>::value_type());
+                typename linalg_traits<MAT>::value_type(),
+                typename linalg_traits<VECT>::value_type());
   }
 
   /* ********************************************************************* */
@@ -280,21 +281,21 @@ namespace gmm {
     if (n <= 2) { q = n; p = 0; }
     else {
       for (size_type i = 1; i < n-q; ++i)
-	if (gmm::abs(A(i,i-1)) < (gmm::abs(A(i,i))+ gmm::abs(A(i-1,i-1)))*tol
-	    || gmm::abs(A(i,i-1)) < rmin)
-	  A(i,i-1) = T(0);
-      
+        if (gmm::abs(A(i,i-1)) < (gmm::abs(A(i,i))+ gmm::abs(A(i-1,i-1)))*tol
+            || gmm::abs(A(i,i-1)) < rmin)
+          A(i,i-1) = T(0);
+
       while ((q < n-1 && A(n-1-q, n-2-q) == T(0)) ||
-	     (q < n-2 && A(n-2-q, n-3-q) == T(0))) ++q;
+             (q < n-2 && A(n-2-q, n-3-q) == T(0))) ++q;
       if (q >= n-2) q = n;
       p = n-q; if (p) --p; if (p) --p;
       while (p > 0 && A(p,p-1) != T(0)) --p;
     }
   }
-  
+
   template <typename MAT, typename Ttol> inline
   void symmetric_qr_stop_criterion(const MAT &AA, size_type &p, size_type &q,
-				Ttol tol) {
+                                Ttol tol) {
     typedef typename linalg_traits<MAT>::value_type T;
     typedef typename number_traits<T>::magnitude_type R;
     R rmin = default_min(R()) * R(2);
@@ -303,10 +304,10 @@ namespace gmm {
     if (n <= 1) { q = n; p = 0; }
     else {
       for (size_type i = 1; i < n-q; ++i)
-	if (gmm::abs(A(i,i-1)) < (gmm::abs(A(i,i))+ gmm::abs(A(i-1,i-1)))*tol
-	    || gmm::abs(A(i,i-1)) < rmin)
-	  A(i,i-1) = T(0);
-      
+        if (gmm::abs(A(i,i-1)) < (gmm::abs(A(i,i))+ gmm::abs(A(i-1,i-1)))*tol
+            || gmm::abs(A(i,i-1)) < rmin)
+          A(i,i-1) = T(0);
+
       while (q < n-1 && A(n-1-q, n-2-q) == T(0)) ++q;
       if (q >= n-1) q = n;
       p = n-q; if (p) --p; if (p) --p;
@@ -316,7 +317,7 @@ namespace gmm {
 
   template <typename VECT1, typename VECT2, typename Ttol> inline
   void symmetric_qr_stop_criterion(const VECT1 &diag, const VECT2 &sdiag_,
-				   size_type &p, size_type &q, Ttol tol) {
+                                   size_type &p, size_type &q, Ttol tol) {
     typedef typename linalg_traits<VECT2>::value_type T;
     typedef typename number_traits<T>::magnitude_type R;
     R rmin = default_min(R()) * R(2);
@@ -325,8 +326,8 @@ namespace gmm {
     if (n <= 1) { q = n; p = 0; return; }
     for (size_type i = 1; i < n-q; ++i)
       if (gmm::abs(sdiag[i-1]) < (gmm::abs(diag[i])+ gmm::abs(diag[i-1]))*tol
-	  || gmm::abs(sdiag[i-1]) < rmin)
-	sdiag[i-1] = T(0);
+          || gmm::abs(sdiag[i-1]) < rmin)
+        sdiag[i-1] = T(0);
     while (q < n-1 && sdiag[n-2-q] == T(0)) ++q;
     if (q >= n-1) q = n;
     p = n-q; if (p) --p; if (p) --p;
@@ -343,30 +344,30 @@ namespace gmm {
     typedef typename number_traits<T>::magnitude_type R;
 
     size_type n = mat_nrows(H), nq = mat_nrows(Q);
+    if (n < 2) return;
     sub_interval SUBQ(0, nq), SUBL(0, 2);
     std::vector<T> v(2), w(std::max(n, nq)); v[0] = T(1);
-    if (n < 2) return;
     tol *= Ttol(2);
     Ttol tol_i = tol * gmm::abs(H(0,0)), tol_cplx = tol_i;
     for (size_type i = 0; i < n-1; ++i) {
       tol_i = (gmm::abs(H(i,i))+gmm::abs(H(i+1,i+1)))*tol;
       tol_cplx = std::max(tol_cplx, tol_i);
-      
+
       if (gmm::abs(H(i+1,i)) > tol_i) { // 2x2 block detected
-	T tr = (H(i+1, i+1) - H(i,i)) / T(2);
-	T delta = tr*tr + H(i,i+1)*H(i+1, i);
-	
-	if (is_complex(T()) || gmm::real(delta) >= R(0)) {
-	  sub_interval SUBI(i, 2);
-	  T theta = (tr - gmm::sqrt(delta)) / H(i+1,i);
-	  R a = gmm::abs(theta);
-	  v[1] = (a == R(0)) ? T(-1)
-	    : gmm::conj(theta) * (R(1) - gmm::sqrt(a*a + R(1)) / a);
-	  row_house_update(sub_matrix(H, SUBI), v, sub_vector(w, SUBL));
-	  col_house_update(sub_matrix(H, SUBI), v, sub_vector(w, SUBL));
-	  col_house_update(sub_matrix(Q, SUBQ, SUBI), v, sub_vector(w, SUBQ));
-	}
-	++i;
+        T tr = (H(i+1, i+1) - H(i,i)) / T(2);
+        T delta = tr*tr + H(i,i+1)*H(i+1, i);
+
+        if (is_complex(T()) || gmm::real(delta) >= R(0)) {
+          sub_interval SUBI(i, 2);
+          T theta = (tr - gmm::sqrt(delta)) / H(i+1,i);
+          R a = gmm::abs(theta);
+          v[1] = (a == R(0)) ? T(-1)
+            : gmm::conj(theta) * (R(1) - gmm::sqrt(a*a + R(1)) / a);
+          row_house_update(sub_matrix(H, SUBI), v, sub_vector(w, SUBL));
+          col_house_update(sub_matrix(H, SUBI), v, sub_vector(w, SUBL));
+          col_house_update(sub_matrix(Q, SUBQ, SUBI), v, sub_vector(w, SUBQ));
+        }
+        ++i;
       }
     }
   }
@@ -385,16 +386,16 @@ namespace gmm {
   // Very slow method. Use implicit_qr_method instead.
   template <typename MAT1, typename VECT, typename MAT2>
     void rudimentary_qr_algorithm(const MAT1 &A, const VECT &eigval_,
-				  const MAT2 &eigvect_,
-				  tol_type_for_qr tol = default_tol_for_qr,
-				  bool compvect = true) {
+                                  const MAT2 &eigvect_,
+                                  tol_type_for_qr tol = default_tol_for_qr,
+                                  bool compvect = true) {
     VECT &eigval = const_cast<VECT &>(eigval_);
     MAT2 &eigvect = const_cast<MAT2 &>(eigvect_);
 
     typedef typename linalg_traits<MAT1>::value_type value_type;
 
     size_type n = mat_nrows(A), p, q = 0, ite = 0;
-    dense_matrix<value_type> Q(n, n), R(n,n), A1(n,n); 
+    dense_matrix<value_type> Q(n, n), R(n,n), A1(n,n);
     gmm::copy(A, A1);
 
     Hessenberg_reduction(A1, eigvect, compvect);
@@ -404,20 +405,20 @@ namespace gmm {
       qr_factor(A1, Q, R);
       gmm::mult(R, Q, A1);
       if (compvect) { gmm::mult(eigvect, Q, R); gmm::copy(R, eigvect); }
-      
+
       qr_stop_criterion(A1, p, q, tol);
       ++ite;
       GMM_ASSERT1(ite < n*1000, "QR algorithm failed");
     }
     if (compvect) block2x2_reduction(A1, Q, tol);
-    extract_eig(A1, eigval, tol); 
+    extract_eig(A1, eigval, tol);
   }
 
   template <typename MAT1, typename VECT>
     void rudimentary_qr_algorithm(const MAT1 &a, VECT &eigval,
-				  tol_type_for_qr tol = default_tol_for_qr) {
+                                  tol_type_for_qr tol = default_tol_for_qr) {
     dense_matrix<typename linalg_traits<MAT1>::value_type> m(0,0);
-    rudimentary_qr_algorithm(a, eigval, m, tol, false); 
+    rudimentary_qr_algorithm(a, eigval, m, tol, false);
   }
 
   /* ********************************************************************* */
@@ -428,8 +429,8 @@ namespace gmm {
     void Francis_qr_step(const MAT1& HH, const MAT2 &QQ, bool compute_Q) {
     MAT1& H = const_cast<MAT1&>(HH); MAT2& Q = const_cast<MAT2&>(QQ);
     typedef typename linalg_traits<MAT1>::value_type value_type;
-    size_type n = mat_nrows(H), nq = mat_nrows(Q); 
-    
+    size_type n = mat_nrows(H), nq = mat_nrows(Q);
+
     std::vector<value_type> v(3), w(std::max(n, nq));
 
     value_type s = H(n-2, n-2) + H(n-1, n-1);
@@ -445,12 +446,12 @@ namespace gmm {
       house_vector(v);
       size_type r = std::min(k+4, n), q = (k==0) ? 0 : k-1;
       sub_interval SUBI(k, 3), SUBJ(0, r), SUBK(q, n-q);
-      
+
       row_house_update(sub_matrix(H, SUBI, SUBK),  v, sub_vector(w, SUBK));
       col_house_update(sub_matrix(H, SUBJ, SUBI),  v, sub_vector(w, SUBJ));
-      
+
       if (compute_Q)
-       	col_house_update(sub_matrix(Q, SUBQ, SUBI),  v, sub_vector(w, SUBQ));
+        col_house_update(sub_matrix(Q, SUBQ, SUBI),  v, sub_vector(w, SUBQ));
 
       x = H(k+1, k); y = H(k+2, k);
       if (k < n-3) z = H(k+3, k);
@@ -471,7 +472,7 @@ namespace gmm {
 
   template <typename MAT1, typename MAT2, typename Ttol>
   void Wilkinson_double_shift_qr_step(const MAT1& HH, const MAT2 &QQ,
-				      Ttol tol, bool exc, bool compute_Q) {
+                                      Ttol tol, bool exc, bool compute_Q) {
     MAT1& H = const_cast<MAT1&>(HH); MAT2& Q = const_cast<MAT2&>(QQ);
     typedef typename linalg_traits<MAT1>::value_type T;
     typedef typename number_traits<T>::magnitude_type R;
@@ -480,7 +481,7 @@ namespace gmm {
     std::vector<T> v(3), w(std::max(n, nq));
     const R dat1(0.75), dat2(-0.4375);
     T h33, h44, h43h34, v1(0), v2(0), v3(0);
-    
+
     if (exc) {                    /* Exceptional shift.                    */
       R s = gmm::abs(H(n-1, n-2)) + gmm::abs(H(n-2, n-3));
       h33 = h44 = dat1 * s;
@@ -517,13 +518,13 @@ namespace gmm {
       house_vector(v);
       size_type r = std::min(k+4, n), q = (k==0) ? 0 : k-1;
       sub_interval SUBI(k, 3), SUBJ(0, r), SUBK(q, n-q);
-      
+
       row_house_update(sub_matrix(H, SUBI, SUBK),  v, sub_vector(w, SUBK));
       col_house_update(sub_matrix(H, SUBJ, SUBI),  v, sub_vector(w, SUBJ));
       if (k > m-1) { H(k+1, k-1) = T(0); if (k < n-3) H(k+2, k-1) = T(0); }
-      
+
       if (compute_Q)
-       	col_house_update(sub_matrix(Q, SUBQ, SUBI),  v, sub_vector(w, SUBQ));
+        col_house_update(sub_matrix(Q, SUBQ, SUBI),  v, sub_vector(w, SUBQ));
 
       v1 = H(k+1, k); v2 = H(k+2, k);
       if (k < n-3) v3 = H(k+3, k);
@@ -543,13 +544,13 @@ namespace gmm {
 
   // QR method for real or complex square matrices based on an
   // implicit QR factorisation. eigval has to be a complex vector
-  // if A has complex eigeinvalues. complexity about 10n^3, 25n^3 if
+  // if A has complex eigenvalues. Complexity about 10n^3, 25n^3 if
   // eigenvectors are computed
   template <typename MAT1, typename VECT, typename MAT2>
     void implicit_qr_algorithm(const MAT1 &A, const VECT &eigval_,
-			       const MAT2 &Q_, 
-			       tol_type_for_qr tol = default_tol_for_qr,
-			       bool compvect = true) {
+                               const MAT2 &Q_,
+                               tol_type_for_qr tol = default_tol_for_qr,
+                               bool compvect = true) {
     VECT &eigval = const_cast<VECT &>(eigval_);
     MAT2 &Q = const_cast<MAT2 &>(Q_);
     typedef typename linalg_traits<MAT1>::value_type value_type;
@@ -561,15 +562,15 @@ namespace gmm {
     gmm::copy(A, H);
     Hessenberg_reduction(H, Q, compvect);
     qr_stop_criterion(H, p, q, tol);
-    
+
     while (q < n) {
       sub_interval SUBI(p, n-p-q), SUBJ(0, mat_ncols(Q));
       if (compvect) SUBK = SUBI;
 //       Francis_qr_step(sub_matrix(H, SUBI),
-// 		      sub_matrix(Q, SUBJ, SUBK), compvect);
-      Wilkinson_double_shift_qr_step(sub_matrix(H, SUBI), 
-				     sub_matrix(Q, SUBJ, SUBK),
-				     tol, (its == 10 || its == 20), compvect);
+//                       sub_matrix(Q, SUBJ, SUBK), compvect);
+      Wilkinson_double_shift_qr_step(sub_matrix(H, SUBI),
+                                     sub_matrix(Q, SUBJ, SUBK),
+                                     tol, (its == 10 || its == 20), compvect);
       q_old = q;
       qr_stop_criterion(H, p, q, tol*2);
       if (q != q_old) its = 0;
@@ -583,18 +584,18 @@ namespace gmm {
 
   template <typename MAT1, typename VECT>
     void implicit_qr_algorithm(const MAT1 &a, VECT &eigval,
-			       tol_type_for_qr tol = default_tol_for_qr) {
+                               tol_type_for_qr tol = default_tol_for_qr) {
     dense_matrix<typename linalg_traits<MAT1>::value_type> m(1,1);
-    implicit_qr_algorithm(a, eigval, m, tol, false); 
+    implicit_qr_algorithm(a, eigval, m, tol, false);
   }
 
   /* ********************************************************************* */
   /*    Implicit symmetric QR step with Wilkinson Shift.                   */
   /* ********************************************************************* */
 
-  template <typename MAT1, typename MAT2> 
+  template <typename MAT1, typename MAT2>
     void symmetric_Wilkinson_qr_step(const MAT1& MM, const MAT2 &ZZ,
-				     bool compute_z) {
+                                     bool compute_z) {
     MAT1& M = const_cast<MAT1&>(MM); MAT2& Z = const_cast<MAT2&>(ZZ);
     typedef typename linalg_traits<MAT1>::value_type T;
     typedef typename number_traits<T>::magnitude_type R;
@@ -603,8 +604,8 @@ namespace gmm {
     for (size_type i = 0; i < n; ++i) {
       M(i, i) = T(gmm::real(M(i, i)));
       if (i > 0) {
-	T a = (M(i, i-1) + gmm::conj(M(i-1, i)))/R(2);
-	M(i, i-1) = a; M(i-1, i) = gmm::conj(a);
+        T a = (M(i, i-1) + gmm::conj(M(i-1, i)))/R(2);
+        M(i, i-1) = a; M(i-1, i) = gmm::conj(a);
       }
     }
 
@@ -633,9 +634,9 @@ namespace gmm {
 
   }
 
-  template <typename VECT1, typename VECT2, typename MAT> 
+  template <typename VECT1, typename VECT2, typename MAT>
   void symmetric_Wilkinson_qr_step(const VECT1& diag_, const VECT2& sdiag_,
-				   const MAT &ZZ, bool compute_z) {
+                                   const MAT &ZZ, bool compute_z) {
     VECT1& diag = const_cast<VECT1&>(diag_);
     VECT2& sdiag = const_cast<VECT2&>(sdiag_);
     MAT& Z = const_cast<MAT&>(ZZ);
@@ -682,8 +683,8 @@ namespace gmm {
       a20 = a31; a21 = a32; a31 = T(0);
 
       if (k < n-1) {
-	sdiag[k] = (gmm::conj(a23) + a32) / R(2);
-	a22 = T(diag[k+1]); a32 = sdiag[k+1]; a23 = gmm::conj(a32);
+        sdiag[k] = (gmm::conj(a23) + a32) / R(2);
+        a22 = T(diag[k+1]); a32 = sdiag[k+1]; a23 = gmm::conj(a32);
       }
     }
   }
@@ -698,9 +699,9 @@ namespace gmm {
   // complexity about 4n^3/3, 9n^3 if eigenvectors are computed
   template <typename MAT1, typename VECT, typename MAT2>
   void symmetric_qr_algorithm_old(const MAT1 &A, const VECT &eigval_,
-			      const MAT2 &eigvect_,
-			      tol_type_for_qr tol = default_tol_for_qr,
-			      bool compvect = true) {
+                              const MAT2 &eigvect_,
+                              tol_type_for_qr tol = default_tol_for_qr,
+                              bool compvect = true) {
     VECT &eigval = const_cast<VECT &>(eigval_);
     MAT2 &eigvect = const_cast<MAT2 &>(eigvect_);
     typedef typename linalg_traits<MAT1>::value_type T;
@@ -712,30 +713,30 @@ namespace gmm {
     gmm::copy(A, Tri);
 
     Householder_tridiagonalization(Tri, eigvect, compvect);
-    
+
     symmetric_qr_stop_criterion(Tri, p, q, tol);
-    
+
     while (q < n) {
 
       sub_interval SUBI(p, n-p-q), SUBJ(0, mat_ncols(eigvect)), SUBK(p, n-p-q);
       if (!compvect) SUBK = sub_interval(0,0);
-      symmetric_Wilkinson_qr_step(sub_matrix(Tri, SUBI), 
-				  sub_matrix(eigvect, SUBJ, SUBK), compvect);
-      
+      symmetric_Wilkinson_qr_step(sub_matrix(Tri, SUBI),
+                                  sub_matrix(eigvect, SUBJ, SUBK), compvect);
+
       symmetric_qr_stop_criterion(Tri, p, q, tol*R(2));
       ++ite;
       GMM_ASSERT1(ite < n*100, "QR algorithm failed. Probably, your matrix"
-		  " is not real symmetric or complex hermitian");
+                  " is not real symmetric or complex hermitian");
     }
-    
+
     extract_eig(Tri, eigval, tol);
   }
 
   template <typename MAT1, typename VECT, typename MAT2>
   void symmetric_qr_algorithm(const MAT1 &A, const VECT &eigval_,
-			      const MAT2 &eigvect_,
-			      tol_type_for_qr tol = default_tol_for_qr,
-			      bool compvect = true) {
+                              const MAT2 &eigvect_,
+                              tol_type_for_qr tol = default_tol_for_qr,
+                              bool compvect = true) {
     VECT &eigval = const_cast<VECT &>(eigval_);
     MAT2 &eigvect = const_cast<MAT2 &>(eigvect_);
     typedef typename linalg_traits<MAT1>::value_type T;
@@ -754,29 +755,29 @@ namespace gmm {
     std::vector<T> sdiag(n);
     for (size_type i = 0; i < n; ++i)
       { diag[i] = gmm::real(Tri(i, i)); if (i+1 < n) sdiag[i] = Tri(i+1, i); }
-    
+
     symmetric_qr_stop_criterion(diag, sdiag, p, q, tol);
-    
+
     while (q < n) {
       sub_interval SUBI(p, n-p-q), SUBJ(0, mat_ncols(eigvect)), SUBK(p, n-p-q);
       if (!compvect) SUBK = sub_interval(0,0);
-      
+
       symmetric_Wilkinson_qr_step(sub_vector(diag, SUBI),
-				  sub_vector(sdiag, SUBI),
-				  sub_matrix(eigvect, SUBJ, SUBK), compvect);
+                                  sub_vector(sdiag, SUBI),
+                                  sub_matrix(eigvect, SUBJ, SUBK), compvect);
 
       symmetric_qr_stop_criterion(diag, sdiag, p, q, tol*R(3));
       ++ite;
       GMM_ASSERT1(ite < n*100, "QR algorithm failed.");
     }
-    
+
     gmm::copy(diag, eigval);
   }
 
 
   template <typename MAT1, typename VECT>
     void symmetric_qr_algorithm(const MAT1 &a, VECT &eigval,
-				tol_type_for_qr tol = default_tol_for_qr) {
+                                tol_type_for_qr tol = default_tol_for_qr) {
     dense_matrix<typename linalg_traits<MAT1>::value_type> m(0,0);
     symmetric_qr_algorithm(a, eigval, m, tol, false);
   }
