@@ -1,32 +1,33 @@
-// -*- c++ -*- (enables emacs c++ mode)
-//===========================================================================
-//
-// Copyright (C) 2002-2008 Yves Renard
-//
-// This file is a part of GETFEM++
-//
-// Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
-// under  the  terms  of the  GNU  Lesser General Public License as published
-// by  the  Free Software Foundation;  either version 2.1 of the License,  or
-// (at your option) any later version.
-// This program  is  distributed  in  the  hope  that it will be useful,  but
-// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-// or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
-// License for more details.
-// You  should  have received a copy of the GNU Lesser General Public License
-// along  with  this program;  if not, write to the Free Software Foundation,
-// Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-// As a special exception, you may use this file as part of a free software
-// library without restriction.  Specifically, if other files instantiate
-// templates or use macros or inline functions from this file, or you compile
-// this file and link it with other files to produce an executable, this
-// file does not by itself cause the resulting executable to be covered by
-// the GNU General Public License.  This exception does not however
-// invalidate any other reasons why the executable file might be covered by
-// the GNU General Public License.
-//
-//===========================================================================
+/* -*- c++ -*- (enables emacs c++ mode) */
+/*===========================================================================
+ 
+ Copyright (C) 2002-2015 Yves Renard
+ 
+ This file is a part of GETFEM++
+ 
+ Getfem++  is  free software;  you  can  redistribute  it  and/or modify it
+ under  the  terms  of the  GNU  Lesser General Public License as published
+ by  the  Free Software Foundation;  either version 3 of the License,  or
+ (at your option) any later version along with the GCC Runtime Library
+ Exception either version 3.1 or (at your option) any later version.
+ This program  is  distributed  in  the  hope  that it will be useful,  but
+ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ or  FITNESS  FOR  A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ License and GCC Runtime Library Exception for more details.
+ You  should  have received a copy of the GNU Lesser General Public License
+ along  with  this program;  if not, write to the Free Software Foundation,
+ Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
+ 
+ As a special exception, you  may use  this file  as it is a part of a free
+ software  library  without  restriction.  Specifically,  if   other  files
+ instantiate  templates  or  use macros or inline functions from this file,
+ or  you compile this  file  and  link  it  with other files  to produce an
+ executable, this file  does  not  by itself cause the resulting executable
+ to be covered  by the GNU Lesser General Public License.  This   exception
+ does not  however  invalidate  any  other  reasons why the executable file
+ might be covered by the GNU Lesser General Public License.
+ 
+===========================================================================*/
 
 /** @file gmm_matrix.h
    @author  Yves Renard <Yves.Renard@insa-lyon.fr>
@@ -63,8 +64,8 @@ namespace gmm
       v1(i,i) += typename linalg_traits<M>::value_type(1);
   }
   template <typename M> inline
-  void add(const identity_matrix &I, const M &v1)
-  { add(I, linalg_const_cast(v1)); }
+  void add(const identity_matrix &II, const M &v1)
+  { add(II, linalg_const_cast(v1)); }
 
   template <typename V1, typename V2> inline
   void mult(const identity_matrix&, const V1 &v1, V2 &v2)
@@ -347,21 +348,26 @@ namespace gmm
     typedef typename std::vector<T>::size_type size_type;
     typedef typename std::vector<T>::iterator iterator;
     typedef typename std::vector<T>::const_iterator const_iterator;
+    typedef typename std::vector<T>::reference reference;
+    typedef typename std::vector<T>::const_reference const_reference;
     
   protected:
     size_type nbc, nbl;
     
   public:
     
-    inline const T& operator ()(size_type l, size_type c) const {
+    inline const_reference operator ()(size_type l, size_type c) const {
       GMM_ASSERT2(l < nbl && c < nbc, "out of range");
       return *(this->begin() + c*nbl+l);
     }
-    inline T& operator ()(size_type l, size_type c) {
+    inline reference operator ()(size_type l, size_type c) {
       GMM_ASSERT2(l < nbl && c < nbc, "out of range");
       return *(this->begin() + c*nbl+l);
     }
 
+    std::vector<T> &as_vector(void) { return *this; }
+    const std::vector<T> &as_vector(void) const { return *this; }
+
     void resize(size_type, size_type);
     void reshape(size_type, size_type);
     
@@ -479,6 +485,7 @@ namespace gmm
   template<typename T> std::ostream &operator <<
     (std::ostream &o, const dense_matrix<T>& m) { gmm::write(o,m); return o; }
 
+
   /* ******************************************************************** */
   /*                                                                      */
   /*	        Read only compressed sparse column matrix                 */
@@ -489,9 +496,9 @@ namespace gmm
   struct csc_matrix {
     typedef unsigned int IND_TYPE;
 
-    T *pr;        // values.
-    IND_TYPE *ir; // row indices.
-    IND_TYPE *jc; // column repartition on pr and ir.
+    std::vector<T> pr;
+    std::vector<IND_TYPE> ir;
+    std::vector<IND_TYPE> jc;
     size_type nc, nr;
 
     typedef T value_type;
@@ -512,16 +519,15 @@ namespace gmm
 
     void init_with_identity(size_type n);
 
-    csc_matrix(void) : pr(0), ir(0), jc(0), nc(0), nr(0) {}
+    csc_matrix(void) :  nc(0), nr(0) {}
     csc_matrix(size_type nnr, size_type nnc);
-    ~csc_matrix() { if (pr) { delete[] pr; delete[] ir; delete[] jc; } }
 
     size_type nrows(void) const { return nr; }
     size_type ncols(void) const { return nc; }
     void swap(csc_matrix<T, shift> &m) { 
       std::swap(pr, m.pr); 
-      std::swap(ir,m.ir); std::swap(jc, m.jc); 
-      std::swap(nc, m.nc); std::swap(nr,m.nr);
+      std::swap(ir, m.ir); std::swap(jc, m.jc); 
+      std::swap(nc, m.nc); std::swap(nr, m.nr);
     }
     value_type operator()(size_type i, size_type j) const
     { return mat_col(*this, j)[i]; }
@@ -530,21 +536,22 @@ namespace gmm
   template <typename T, int shift> template<typename Matrix>
   void csc_matrix<T, shift>::init_with_good_format(const Matrix &B) {
     typedef typename linalg_traits<Matrix>::const_sub_col_type col_type;
-    if (pr) { delete[] pr; delete[] ir; delete[] jc; }
     nc = mat_ncols(B); nr = mat_nrows(B);
-    jc = new IND_TYPE[nc+1];
+    jc.resize(nc+1);
     jc[0] = shift;
     for (size_type j = 0; j < nc; ++j) {
-      jc[j+1] = jc[j] + nnz(mat_const_col(B, j));
+      jc[j+1] = IND_TYPE(jc[j] + nnz(mat_const_col(B, j)));
     }
-    pr = new T[jc[nc]];
-    ir = new IND_TYPE[jc[nc]];
+    pr.resize(jc[nc]);
+    ir.resize(jc[nc]);
     for (size_type j = 0; j < nc; ++j) {
       col_type col = mat_const_col(B, j);
       typename linalg_traits<col_type>::const_iterator
 	it = vect_const_begin(col), ite = vect_const_end(col);
-      for (size_type k = 0; it != ite; ++it, ++k)
-	{ pr[jc[j]-shift+k] = *it; ir[jc[j]-shift+k] = it.index() + shift; }
+      for (size_type k = 0; it != ite; ++it, ++k) {
+	pr[jc[j]-shift+k] = *it;
+	ir[jc[j]-shift+k] = IND_TYPE(it.index() + shift);
+      }
     }
   }
   
@@ -557,11 +564,8 @@ namespace gmm
   
   template <typename T, int shift>
   void csc_matrix<T, shift>::init_with_identity(size_type n) {
-    if (pr) { delete[] pr; delete[] ir; delete[] jc; }
     nc = nr = n; 
-    pr = new T[nc];
-    ir = new IND_TYPE[nc];
-    jc = new IND_TYPE[nc+1];
+    pr.resize(nc); ir.resize(nc); jc.resize(nc+1);
     for (size_type j = 0; j < nc; ++j)
       { ir[j] = jc[j] = shift + j; pr[j] = T(1); }
     jc[nc] = shift + nc;
@@ -570,8 +574,7 @@ namespace gmm
   template <typename T, int shift>
   csc_matrix<T, shift>::csc_matrix(size_type nnr, size_type nnc)
     : nc(nnc), nr(nnr) {
-    pr = new T[1];  ir = new IND_TYPE[1];
-    jc = new IND_TYPE[nc+1];
+    pr.resize(1);  ir.resize(1); jc.resize(nc+1);
     for (size_type j = 0; j <= nc; ++j) jc[j] = shift;
   }
 
@@ -601,15 +604,17 @@ namespace gmm
     static size_type nrows(const this_type &m) { return m.nrows(); }
     static size_type ncols(const this_type &m) { return m.ncols(); }
     static const_col_iterator col_begin(const this_type &m)
-    { return const_col_iterator(m.pr, m.ir, m.jc, m.nr, m.pr); }
-    static const_col_iterator col_end(const this_type &m)
-    { return const_col_iterator(m.pr, m.ir, m.jc + m.nc, m.nr, m.pr); }
+    { return const_col_iterator(&m.pr[0],&m.ir[0],&m.jc[0], m.nr, &m.pr[0]); }
+    static const_col_iterator col_end(const this_type &m) {
+      return const_col_iterator(&m.pr[0],&m.ir[0],&m.jc[0]+m.nc,
+				m.nr,&m.pr[0]);
+    }
     static const_sub_col_type col(const const_col_iterator &it) {
       return const_sub_col_type(it.pr + *(it.jc) - shift,
 				it.ir + *(it.jc) - shift,
 				*(it.jc + 1) - *(it.jc), it.n);
     }
-    static const origin_type* origin(const this_type &m) { return m.pr; }
+    static const origin_type* origin(const this_type &m) { return &m.pr[0]; }
     static void do_clear(this_type &m) { m.do_clear(); }
     static value_type access(const const_col_iterator &itcol, size_type j)
     { return col(itcol)[j]; }
@@ -639,9 +644,9 @@ namespace gmm
 
     typedef unsigned int IND_TYPE;
 
-    T *pr;        // values.
-    IND_TYPE *ir; // col indices.
-    IND_TYPE *jc; // row repartition on pr and ir.
+    std::vector<T> pr;        // values.
+    std::vector<IND_TYPE> ir; // col indices.
+    std::vector<IND_TYPE> jc; // row repartition on pr and ir.
     size_type nc, nr;
 
     typedef T value_type;
@@ -663,9 +668,8 @@ namespace gmm
     template <typename Matrix> void init_with(const Matrix &A);
     void init_with_identity(size_type n);
 
-    csr_matrix(void) : pr(0), ir(0), jc(0), nc(0), nr(0) {}
+    csr_matrix(void) : nc(0), nr(0) {}
     csr_matrix(size_type nnr, size_type nnc);
-    ~csr_matrix() { if (pr) { delete[] pr; delete[] ir; delete[] jc; } }
 
     size_type nrows(void) const { return nr; }
     size_type ncols(void) const { return nc; }
@@ -682,21 +686,22 @@ namespace gmm
   template <typename T, int shift> template <typename Matrix>
   void csr_matrix<T, shift>::init_with_good_format(const Matrix &B) {
     typedef typename linalg_traits<Matrix>::const_sub_row_type row_type;
-    if (pr) { delete[] pr; delete[] ir; delete[] jc; }
     nc = mat_ncols(B); nr = mat_nrows(B);
-    jc = new IND_TYPE[nr+1];
+    jc.resize(nr+1);
     jc[0] = shift;
     for (size_type j = 0; j < nr; ++j) {
-      jc[j+1] = jc[j] + nnz(mat_const_row(B, j));
+      jc[j+1] = IND_TYPE(jc[j] + nnz(mat_const_row(B, j)));
     }
-    pr = new T[jc[nr]];
-    ir = new IND_TYPE[jc[nr]];
+    pr.resize(jc[nr]);
+    ir.resize(jc[nr]);
     for (size_type j = 0; j < nr; ++j) {
       row_type row = mat_const_row(B, j);
       typename linalg_traits<row_type>::const_iterator
 	it = vect_const_begin(row), ite = vect_const_end(row);
-      for (size_type k = 0; it != ite; ++it, ++k)
-	{ pr[jc[j]-shift+k] = *it; ir[jc[j]-shift+k] = it.index()+shift; }
+      for (size_type k = 0; it != ite; ++it, ++k) {
+	pr[jc[j]-shift+k] = *it;
+	ir[jc[j]-shift+k] = IND_TYPE(it.index()+shift);
+      }
     }
   }
 
@@ -709,11 +714,8 @@ namespace gmm
 
   template <typename T, int shift> 
   void csr_matrix<T, shift>::init_with_identity(size_type n) {
-    if (pr) { delete[] pr; delete[] ir; delete[] jc; }
     nc = nr = n; 
-    pr = new T[nr];
-    ir = new IND_TYPE[nr];
-    jc = new IND_TYPE[nr+1];
+    pr.resize(nr); ir.resize(nr); jc.resize(nr+1);
     for (size_type j = 0; j < nr; ++j)
       { ir[j] = jc[j] = shift + j; pr[j] = T(1); }
     jc[nr] = shift + nr;
@@ -722,8 +724,7 @@ namespace gmm
   template <typename T, int shift>
   csr_matrix<T, shift>::csr_matrix(size_type nnr, size_type nnc)
     : nc(nnc), nr(nnr) {
-    pr = new T[1];  ir = new IND_TYPE[1];
-    jc = new IND_TYPE[nr+1];
+    pr.resize(1);  ir.resize(1); jc.resize(nr+1);
     for (size_type j = 0; j < nr; ++j) jc[j] = shift;
     jc[nr] = shift;
   }
@@ -755,15 +756,15 @@ namespace gmm
     static size_type nrows(const this_type &m) { return m.nrows(); }
     static size_type ncols(const this_type &m) { return m.ncols(); }
     static const_row_iterator row_begin(const this_type &m)
-    { return const_row_iterator(m.pr, m.ir, m.jc, m.nc, m.pr); }
+    { return const_row_iterator(&m.pr[0], &m.ir[0], &m.jc[0], m.nc, &m.pr[0]); }
     static const_row_iterator row_end(const this_type &m)
-    { return const_row_iterator(m.pr, m.ir, m.jc + m.nr, m.nc, m.pr); }
+    { return const_row_iterator(&m.pr[0], &m.ir[0], &m.jc[0] + m.nr, m.nc, &m.pr[0]); }
     static const_sub_row_type row(const const_row_iterator &it) {
       return const_sub_row_type(it.pr + *(it.jc) - shift,
 				it.ir + *(it.jc) - shift,
 				*(it.jc + 1) - *(it.jc), it.n);
     }
-    static const origin_type* origin(const this_type &m) { return m.pr; }
+    static const origin_type* origin(const this_type &m) { return &m.pr[0]; }
     static void do_clear(this_type &m) { m.do_clear(); }
     static value_type access(const const_row_iterator &itrow, size_type j)
     { return row(itrow)[j]; }
@@ -945,9 +946,11 @@ namespace gmm
   /* ******************************************************************** */
 
 #ifdef GMM_USES_MPI
-#include <mpi.h>
- 
-namespace gmm { 
+# include <mpi.h>
+
+namespace gmm {
+
+  
   
   template <typename T> inline MPI_Datatype mpi_type(T)
   { GMM_ASSERT1(false, "Sorry unsupported type"); return MPI_FLOAT; }
@@ -960,13 +963,8 @@ namespace gmm {
 #endif
   inline MPI_Datatype mpi_type(int) { return MPI_INT; }
   inline MPI_Datatype mpi_type(unsigned int) { return MPI_UNSIGNED; }
-  inline MPI_Datatype mpi_type(size_t) {
-    if (sizeof(int) == sizeof(size_t)) return MPI_UNSIGNED;
-    if (sizeof(long) == sizeof(size_t)) return MPI_UNSIGNED_LONG;
-    return MPI_LONG_LONG;
-  }
-
-
+  inline MPI_Datatype mpi_type(long) { return MPI_LONG; }
+  inline MPI_Datatype mpi_type(unsigned long) { return MPI_UNSIGNED_LONG; }
 
   template <typename MAT> struct mpi_distributed_matrix {
     MAT M;
